    // **********************************************************
// ******************   OpenBee Code      *******************
// ***  OpenBee Designed by Melih Karakelle on 2012       ***
// **              an Arudino based RF system              **
// **       This Source code licensed under GPL            **
// **********************************************************
// Version Number     : 1.01
// Latest Code Update : 2012-10-05
// Supported Hardware : OpenBee boards (store.flytron.com)
// Project Forum      : http://forum.flytron.com/
// Google Code Page   : http://code.google.com/p/openbee/
// **********************************************************

// ******************** OpenBee DEVELOPERS ****************** 
// Melih Karakelle (http://www.flytron.com) (forum nick name: Flytron)

#include "config.h"

#include <EEPROM.h>

// **********************************************************
// **                   OpenLRS Functions                  **
// **       This Source code licensed under GPL            **
// **********************************************************
// Latest Code Update : 2012-02-14
// Supported Hardware : OpenLRS Tx boards (M1 & M2) (store.flytron.com)
// Project Forum      : http://forum.flytron.com/viewforum.php?f=7
// Google Code Page   : http://code.google.com/p/openlrs/
// **********************************************************

// **********************************************************
// **      RFM22B/Si4432 control functions for OpenLRS     **
// **       This Source code licensed under GPL            **
// **********************************************************
// Latest Code Update : 2011-09-26
// Supported Hardware : OpenLRS Tx/Rx boards (store.flytron.com)
// Project Forum      : http://forum.flytron.com/viewforum.php?f=7
// Google Code Page   : http://code.google.com/p/openlrs/
// **********************************************************
 
#define NOP() __asm__ __volatile__("nop") 

 
#define RF22B_PWRSTATE_READY    01 
#define RF22B_PWRSTATE_TX        0x09 


#define RF22B_PWRSTATE_RX       05 
#define RF22B_Rx_packet_received_interrupt   0x02 

#define RF22B_PACKET_SENT_INTERRUPT  04 
#define RF22B_PWRSTATE_POWERDOWN  00 

 
unsigned char ItStatus1, ItStatus2; 
 
 
typedef struct   
{ 
 unsigned char reach_1s    : 1; 
} FlagType; 
FlagType               Flag; 
 
//***************************************************************************** 
//***************************************************************************** 

//-------------------------------------------------------------- 
void Write0( void ) 
{ 
    SCK_off;  
    NOP(); 
     
    SDI_off; 
    NOP(); 
     
    SCK_on;  
    NOP(); 
} 
//-------------------------------------------------------------- 
void Write1( void ) 
{ 
    SCK_off;
    NOP(); 
     
    SDI_on;
    NOP(); 
     
    SCK_on; 
    NOP(); 
} 
//-------------------------------------------------------------- 
void Write8bitcommand(unsigned char command)    // keep sel to low 
{ 
 unsigned char n=8; 
    nSEL_on;
    SCK_off;
    nSEL_off; 
    while(n--) 
    { 
         if(command&0x80) 
          Write1(); 
         else 
          Write0();    
              command = command << 1; 
    } 
    SCK_off;
}  


//-------------------------------------------------------------- 
unsigned char _spi_read(unsigned char address) 
{ 
 unsigned char result; 
 send_read_address(address); 
 result = read_8bit_data();  
 nSEL_on; 
 return(result); 
}  

//-------------------------------------------------------------- 
void _spi_write(unsigned char address, unsigned char data) 
{ 
 address |= 0x80; 
 Write8bitcommand(address); 
 send_8bit_data(data);  
 nSEL_on;
}  


//-------Defaults 38.400 baud---------------------------------------------- 
void RF22B_init_parameter(void) 
{ 
 ItStatus1 = _spi_read(0x03); // read status, clear interrupt   
 ItStatus2 = _spi_read(0x04); 
  _spi_write(0x06, 0x00);    // no wakeup up, lbd, 
  _spi_write(0x07, RF22B_PWRSTATE_READY);      // disable lbd, wakeup timer, use internal 32768,xton = 1; in ready mode 
  _spi_write(0x09, 0x7f);  // c = 12.5p   
  _spi_write(0x0a, 0x05); 
  _spi_write(0x0b, 0x12);    // gpio0 TX State
  _spi_write(0x0c, 0x15);    // gpio1 RX State 
  //  -- Old PCB --
  //  _spi_write(0x0b, 0x15);    // gpio0 RX State
  //  _spi_write(0x0c, 0x12);    // gpio1 TX State 
  _spi_write(0x0d, 0xfd);    // gpio 2 micro-controller clk output 
  _spi_write(0x0e, 0x00);    // gpio    0, 1,2 NO OTHER FUNCTION. 
  
  _spi_write(0x70, 0x00);    // disable manchest 
  
   
  /* // 38.4Kbps data rate
  _spi_write(0x6e, 0x09); //case RATE_384K 
  _spi_write(0x6f, 0xD5); //case RATE_384K
  
  _spi_write(0x1c, 0x02); // case RATE_384K
  _spi_write(0x20, 0x68);//  0x20 calculate from the datasheet= 500*(1+2*down3_bypass)/(2^ndec*RB*(1+enmanch)) 
  _spi_write(0x21, 0x01); // 0x21 , rxosr[10--8] = 0; stalltr = (default), ccoff[19:16] = 0; 
  _spi_write(0x22, 0x3A); // 0x22    ncoff =5033 = 0x13a9 
  _spi_write(0x23, 0x93); // 0x23 
  _spi_write(0x24, 0x02); // 0x24 
  _spi_write(0x25, 0x6B); // 0x25 
  _spi_write(0x2a, 0x1e); 
  */
    
  // 57.6Kbps data rate
  _spi_write(0x1c, 0x05); // case RATE_57.6K
  _spi_write(0x20, 0x45);//  0x20 calculate from the datasheet= 500*(1+2*down3_bypass)/(2^ndec*RB*(1+enmanch)) 
  _spi_write(0x21, 0x01); // 0x21 , rxosr[10--8] = 0; stalltr = (default), ccoff[19:16] = 0; 
  _spi_write(0x22, 0xD7); // 0x22    ncoff =5033 = 0x13a9 
  _spi_write(0x23, 0xDC); // 0x23 
  _spi_write(0x24, 0x03); // 0x24 
  _spi_write(0x25, 0xB8); // 0x25 
  _spi_write(0x2a, 0x1e); 
  
  _spi_write(0x6e, 0x0E); //case RATE_57.6K 
  _spi_write(0x6f, 0xBF); //case RATE_57.6K 
  

  _spi_write(0x30, 0x8c);    // enable packet handler, msb first, enable crc, 

  _spi_write(0x32, 0xf3);    // 0x32address enable for headere byte 0, 1,2,3, receive header check for byte 0, 1,2,3 
  _spi_write(0x33, 0x42);    // header 3, 2, 1,0 used for head length, fixed packet length, synchronize word length 3, 2, 
  _spi_write(0x34, 0x07);    // 7 default value or   // 64 nibble = 32byte preamble 
  _spi_write(0x36, 0x2d);    // synchronize word 
 _spi_write(0x37, 0xd4); 
 _spi_write(0x38, 0x00); 
 _spi_write(0x39, 0x00); 
 _spi_write(0x3a, RF_Header[0]);    // tx header 
 _spi_write(0x3b, RF_Header[1]); 
 _spi_write(0x3c, RF_Header[2]); 
 _spi_write(0x3d, RF_Header[3]); 
 _spi_write(0x3e, RF_PACK_SIZE);    // total tx 17 byte 
 
  
  
    //RX HEADER
 _spi_write(0x3f, RF_Header[0]);   // check hearder 
 _spi_write(0x40, RF_Header[1]); 
 _spi_write(0x41, RF_Header[2]); 
 _spi_write(0x42, RF_Header[3]); 
 _spi_write(0x43, 0xff);    // all the bit to be checked 
 _spi_write(0x44, 0xff);    // all the bit to be checked 
 _spi_write(0x45, 0xff);    // all the bit to be checked 
 _spi_write(0x46, 0xff);    // all the bit to be checked 
  

  
  _spi_write(0x6d, 0x07); // 7 set power max power 
  _spi_write(0x79, 0x00);    // no hopping 
  _spi_write(0x7a, HOPPING_STEP_SIZE);    // 60khz step size (10khz x value) // no hopping 

  _spi_write(0x71, 0x23); // Gfsk, fd[8] =0, no invert for Tx/Rx data, fifo mode, txclk -->gpio 
  //_spi_write(0x72, 0x1F); // frequency deviation setting to 19.6khz (for 38.4kbps)
  _spi_write(0x72, 0x2E); // frequency deviation setting to 28.8khz(for 57.6kbps)
  _spi_write(0x73, 0x00);   
  _spi_write(0x74, 0x00);    // no offset 
 

  //band 435.000
 
 _spi_write(0x75, 0x53);    
 _spi_write(0x76, 0x7D);    
 _spi_write(0x77, 0x00); 
}
/*
void fast_init(void){

ItStatus1 = _spi_read(0x03); // read status, clear interrupt   
 ItStatus2 = _spi_read(0x04); 
  _spi_write(0x06, 0x00);    // no wakeup up, lbd, 
  _spi_write(0x07, RF22B_PWRSTATE_READY);      // disable lbd, wakeup timer, use internal 32768,xton = 1; in ready mode 
  _spi_write(0x09, 0x7f);  // c = 12.5p   
  _spi_write(0x0a, 0x05); 
  _spi_write(0x0b, 0x12);    // gpio0 TX State
  _spi_write(0x0c, 0x15);    // gpio RX State 
  //  -- Old PCB --
  //  _spi_write(0x0b, 0x15);    // gpio0 RX State
  //  _spi_write(0x0c, 0x12);    // gpio TX State 
  _spi_write(0x0d, 0xfd);    // gpio 2 micro-controller clk output 
  _spi_write(0x0e, 0x00);    // gpio    0, 1,2 NO OTHER FUNCTION. 
  
  


  _spi_write(0x70, 0x00);    // disable manchest 
  
  _spi_write(0x6e, 0x09); //case RATE_384K: // 38.4k 
  _spi_write(0x6f, 0xD5); //case RATE_384K: // 38.4k 


  _spi_write(0x1c, 0x02); // RATE_24K: // 2.4k
  _spi_write(0x20, 0x68);//0x20 calculate from the datasheet= 500*(1+2*down3_bypass)/(2^ndec*RB*(1+enmanch)) 
  _spi_write(0x21, 0x01); // 0x21 , rxosr[10--8] = 0; stalltr = (default), ccoff[19:16] = 0; 
  _spi_write(0x22, 0x3A); // 0x22    ncoff =5033 = 0x13a9 
  _spi_write(0x23, 0x93); // 0x23 
  _spi_write(0x24, 0x02); // 0x24 
  _spi_write(0x25, 0x6B); // 0x25 
  _spi_write(0x2a, 0x1e); 



}

/**/

//-------------------------------------------------------------- 
void send_read_address(unsigned char i) 
{ 
 i &= 0x7f; 
  
 Write8bitcommand(i); 
}  
//-------------------------------------------------------------- 
void send_8bit_data(unsigned char i) 
{ 
  unsigned char n = 8; 
  SCK_off;
    while(n--) 
    { 
         if(i&0x80) 
          Write1(); 
         else 
          Write0();    
         i = i << 1; 
    } 
   SCK_off;
}  
//-------------------------------------------------------------- 

unsigned char read_8bit_data(void) 
{ 
  unsigned char Result, i; 
  
 SCK_off;
 Result=0; 
    for(i=0;i<8;i++) 
    {                    //read fifo data byte 
       Result=Result<<1; 
       SCK_on;
       NOP(); 
       if(SDO_1) 
       { 
         Result|=1; 
       } 
       SCK_off;
       NOP(); 
     } 
    return(Result); 
}  
//-------------------------------------------------------------- 

//----------------------------------------------------------------------- 
void rx_reset(void) 
{ 
  _spi_write(0x07, RF22B_PWRSTATE_READY); 
  _spi_write(0x7e, 36);    // threshold for rx almost full, interrupt when 1 byte received 
  _spi_write(0x08, 0x03);    //clear fifo disable multi packet 
  _spi_write(0x08, 0x00);    // clear fifo, disable multi packet 
  _spi_write(0x07,RF22B_PWRSTATE_RX );  // to rx mode 
  _spi_write(0x05, RF22B_Rx_packet_received_interrupt); 
  ItStatus1 = _spi_read(0x03);  //read the Interrupt Status1 register 
  ItStatus2 = _spi_read(0x04);  
}  
//-----------------------------------------------------------------------    

void to_rx_mode(void) 
{  
 to_ready_mode(); 
 delay(50); 
 rx_reset(); 
 NOP(); 
}  

//-------------------------------------------------------------- 
void to_tx_mode(void) 
{ 
 
 unsigned char i;
 to_ready_mode();
 
  _spi_write(0x08, 0x03);    // disABLE AUTO TX MODE, enable multi packet clear fifo 
  _spi_write(0x08, 0x00);    // disABLE AUTO TX MODE, enable multi packet, clear fifo 
  
    // ph +fifo mode 
  _spi_write(0x34, 7);     // 64 nibble = 32byte preamble 
  _spi_write(0x3e, RF_PACK_SIZE);    // total tx 17 byte 

 for (i = 0; i<RF_PACK_SIZE; i++)
 { 
  _spi_write(0x7f, RF_Tx_Buffer[i]); 
 } 
 
 
 _spi_write(0x05, RF22B_PACKET_SENT_INTERRUPT);  
 ItStatus1 = _spi_read(0x03);      //read the Interrupt Status1 register 
 ItStatus2 = _spi_read(0x04); 
  _spi_write(0x07, RF22B_PWRSTATE_TX);    // to tx mode 

  while(nIRQ_1);

  to_ready_mode();

}  
//-------------------------------------------------------------- 
void to_ready_mode(void) 
{ 
 ItStatus1 = _spi_read(0x03);   
 ItStatus2 = _spi_read(0x04); 
 _spi_write(0x07, RF22B_PWRSTATE_READY); 
}  
/*-------------------------------------------------------------- 
void to_sleep_mode(void) 
{ 
  _spi_write(0x07, RF22B_PWRSTATE_READY);  
   
  ItStatus1 = _spi_read(0x03);  //read the Interrupt Status1 register 
  ItStatus2 = _spi_read(0x04);    
  _spi_write(0x07, RF22B_PWRSTATE_POWERDOWN); 

} 
/**/
//--------------------------------------------------------------   
  
  
void frequency_configurator(long frequency){
  
  // frequency formulation from Si4432 chip's datasheet
  // original formulation is working with mHz values and floating numbers, I replaced them with kHz values.
  
  unsigned int frequency_constant = (frequency/10000) - 24; // result is 19 for 430-439.990 Mhz

  frequency = frequency / 10;
  frequency = frequency - 24000;
  
  frequency =  frequency - (frequency_constant*1000);  
  
  frequency = frequency * 64; // this is the Nominal Carrier Frequency (fc) value for register setting
  
   
  byte byte0 = (byte) frequency;
  byte byte1 = (byte) (frequency >> 8);
  
  _spi_write(0x75, 0x40 + frequency_constant);
  _spi_write(0x76, byte1);    
  _spi_write(0x77, byte0); 

}


void RFM22B_Int()
{
 if (RF_Mode == Transmit) 
    {
     RF_Mode = Receive; 
    } 
 if (RF_Mode == Receive) 
    {
     RF_Mode = Received; 
    }  
}

/*############# RED LED BLINK #################
void Red_LED_Blink(unsigned short blink_count)
  {
  unsigned char i;
  for (i=0;i<blink_count;i++)
     {
     delay(100);
     Red_LED_ON;
     delay(100);
     Red_LED_OFF;
     }
  }
  
/*############# GREEN LED BLINK #################
void Blue_LED_Blink(unsigned short blink_count)
  {
  unsigned char i;
  for (i=0;i<blink_count;i++)
     {
     delay(100);
     Blue_LED_ON;
     delay(100);
     Blue_LED_OFF;
     }
  }  

/*############# FREQUENCY HOPPING #################
#if (FREQUENCY_HOPPING==1)
void Hopping(void){

hopping_channel++;
if (hopping_channel>2) hopping_channel = 0;

_spi_write(0x79, hop_list[hopping_channel]);

    #if (DEBUG_MODE == 5)
      Serial.println(int(hop_list[hopping_channel]));
    #endif  

}
#endif
/**/
//############# RF POWER SETUP #################
void Power_Set(unsigned short level)
{
  //Power Level value between 0-7
  //0 = +1 dBm
  //1 = +2 dBmf
  //2 = +5 dBm
  //3 = +8 dBm
  //4 = +11 dBm
  //5 = +14 dBm
  //6 = +17 dBm
  //7 = +20 dB 
  if (level<8) _spi_write(0x6d, level);  
  
}

void ClearBuffers(void)
{
  for(unsigned char i = 0; i<RF_PACK_SIZE; i++) {
   RF_Tx_Buffer[i] = 0;
   RF_Rx_Buffer[i] = 0;
  }
}

struct Ack {
  unsigned char signature;
  unsigned int package_size;
  unsigned char data[14];
};

struct Ack_req {
  unsigned char signature;
  unsigned int low_mask;
  unsigned int high_mask;
  unsigned char padding[12];
};

struct Data {
  unsigned char signature;
  unsigned char data[16];
};

struct Fin {
  unsigned char signature;
  unsigned int more_size;
  unsigned char padding[14];
};

#define MAX_TX_BUFF_SIZE 512

unsigned char tx_buff[MAX_TX_BUFF_SIZE];
unsigned int buff_head_ptr=0;
unsigned int buff_read_ptr = 0;

//######## TELEMETRY TRANSPARENT BRIDGE #########

void updateTXBuffer() {
  while (Serial.available()>0) {
    tx_buff[buff_head_ptr] = Serial.read();
    buff_head_ptr++;
    if (buff_head_ptr==MAX_TX_BUFF_SIZE) {
      buff_head_ptr=0;
    }
  }
}

boolean hasTXData() {
  return buff_head_ptr!=buff_read_ptr;
}

unsigned int getTXBuffSize() {
  if (!hasTXData()) return 0;
  if (buff_head_ptr>buff_read_ptr) return buff_head_ptr-buff_read_ptr;
  return buff_head_ptr+ (MAX_TX_BUFF_SIZE-buff_read_ptr);
}

unsigned char getTXbyte() {
  if (hasTXData()) {
    unsigned char data = tx_buff[buff_read_ptr];
    buff_read_ptr++;
    if (buff_read_ptr==MAX_TX_BUFF_SIZE) {
      buff_read_ptr=0;
    }
    return data;
  }
  return 0;  // Send 0s if people ask
}

#define ACKK 0
#define ACK_REQ 1
#define DATA 2
#define FIN 3

#define FRAME_SIZE RF_PACK_SIZE-1

unsigned char createSignature(unsigned char type, unsigned char data) {
  return (data<<2) + type;
}

unsigned char createByte(boolean bit1,boolean bit2=0,boolean bit3=0,boolean bit4=0,boolean bit5=0,boolean bit6=0,boolean bit7=0,boolean bit8=0) {
  unsigned char retByte = 0;
  if (bit1) retByte+=1;
  if (bit2) retByte+=2;
  if (bit3) retByte+=4;
  if (bit4) retByte+=8;
  if (bit5) retByte+=16;
  if (bit6) retByte+=32;
  if (bit7) retByte+=64;
  if (bit8) retByte+=128;
  return retByte;
}

boolean master=false;

#define MAX_WAIT_CHECKS 100;
int waiting = 0;

unsigned char currentState = ACKK;

void rdyToTX() {
  updateTXBuffer();
  if (waiting>0) {
    waiting--;
    delay(1);
  }
  if (waiting==0 && (currentState!=ACKK || hasTXData())) transmit();
}

byte retryCount = 0;
void transmit() {
  switch (currentState) {
  case ACKK:
    sendAck();
    break;
  case ACK_REQ:
    sendAckReq();
    break;
  case FIN:
    if (retryCount<3) {
      retryCount++;
      sendFin(master,hasTXData() );
      waiting = MAX_WAIT_CHECKS;
    }
    else {
      currentState = ACKK;
      retryCount = 0;
    }
    break;
  }
}

Ack ackPacket = {0,0};
Ack_req ackReqPacket = {0,0,0};

void byteTransfer() {
  Red_LED_ON;
  RF_Mode = Transmit;
  to_tx_mode();
  rx_reset();
  
  RF_Tx_Buffer[0] = 0;
  RF_Tx_Buffer[1] = 0;
  RF_Tx_Buffer[2] = 0;
  RF_Tx_Buffer[3] = 0;
  RF_Tx_Buffer[4] = 0;
  RF_Tx_Buffer[5] = 0;
  RF_Tx_Buffer[6] = 0;
  RF_Tx_Buffer[7] = 0;
  RF_Tx_Buffer[8] = 0;
  RF_Tx_Buffer[9] = 0;
  RF_Tx_Buffer[10] = 0;
  RF_Tx_Buffer[11] = 0;
  RF_Tx_Buffer[12] = 0;
  RF_Tx_Buffer[13] = 0;
  RF_Tx_Buffer[14] = 0;
  RF_Tx_Buffer[15] = 0;
  RF_Tx_Buffer[16] = 0;
  RF_Mode = Receive;
  Red_LED_OFF;
}

boolean interrupted = false;

void sendAck() {
  if (!hasTXData() && !interrupted) return;
  
  if (!interrupted) {
    ackPacket.package_size = getTXBuffSize();
    byte ack_bytes = ackPacket.package_size;
    if (ack_bytes>14) ack_bytes = 14;
    for (int i=0;i<ack_bytes;i++) {
      ackPacket.data[i] = getTXbyte();
    }
    ackPacket.signature = createSignature(ACKK,( ((int)ackPacket.package_size)-ack_bytes+FRAME_SIZE-1)/(FRAME_SIZE));
  }

  if (RF_Mode == Received) {
    interrupted = true;
    return;  // In the middle of receiving from another unit
  }
  interrupted = false;
  
  master = true;
  
  memcpy(RF_Tx_Buffer,&ackPacket,RF_PACK_SIZE);

  byteTransfer();
  waiting = MAX_WAIT_CHECKS;
}

Data dataPackets[32];
boolean dataFree = true;

unsigned int expectedPackets = 0;

void sendAckReq() {
  master = false;
  unsigned int low_mask = 0;
  unsigned int high_mask = 0;
  if (ackReqPacket.signature==0) {
    ackPacket.signature = 0;  // we aren't prepping to send anything.
    ackReqPacket.signature = createSignature(ACK_REQ,expectedPackets);
    if (expectedPackets>16) {
      low_mask = 0xFF;
      for (int i=16;i<expectedPackets;i++) {
        high_mask<<1;
        high_mask++;
      }
    }
    else {
      for (int i=0;i<expectedPackets;i++) {
        low_mask<<1;
        low_mask++;
      }
    }
  }
  else {
    unsigned int bitMask = 1;
    for (int j=0;j<expectedPackets;j++) {
      if (dataPackets[j].signature==0) {
        if (j<16) low_mask = low_mask + bitMask;
        else high_mask = high_mask + bitMask;
      }
      bitMask = bitMask<<1;
      if (bitMask==0) bitMask = 1;  // if it rolls out, going to high_mask
    }
  }
  
  if (dataFree) {
    dataFree = false;
    for (byte packetOn = 0;packetOn<expectedPackets;packetOn++) {
          dataPackets[packetOn].signature = 0;
    }
  }
  ackReqPacket.low_mask = low_mask;
  ackReqPacket.high_mask = high_mask;
  
  memcpy(RF_Tx_Buffer,&ackReqPacket,RF_PACK_SIZE);
  byteTransfer();
  waiting = MAX_WAIT_CHECKS;
  waiting = waiting>>2;  // wait 1/4 as long
}

void sendDataPacket(byte packetNum) {
  memcpy(RF_Tx_Buffer,&dataPackets[packetNum],RF_PACK_SIZE);
  byteTransfer();
}

void sendFin(boolean master, boolean hasMore) {
  Fin finPacket = {createSignature(FIN,createByte(master,hasMore)),getTXBuffSize()};
  memcpy(RF_Tx_Buffer,&finPacket,RF_PACK_SIZE);
  byteTransfer();
}

void readPacket() {
  unsigned char signature = RF_Rx_Buffer[0] & 3;
  switch (signature) {
    case ACKK:
      readAck();
      break;
    case ACK_REQ:
      readAckReq();
      break;
    case DATA:
      readData();
      break;
    case FIN:
      readFin();
  }
  if (waiting>0) waiting = MAX_WAIT_CHECKS;
}

unsigned int reading_length=0;

void readAck() {
  Ack readAck = {};
  memcpy(&readAck,RF_Rx_Buffer, RF_PACK_SIZE);
  if (readAck.package_size>ackPacket.package_size || (readAck.package_size==ackPacket.package_size && readAck.signature>ackPacket.signature) ) {
    reading_length = readAck.package_size;
    int i;
    for (i=0;i<sizeof(readAck.data) && i<reading_length;i++)
      Serial.write(readAck.data[i]);
      
    reading_length-=i;  //  we already read i bytes.
    
    currentState = ACK_REQ;

    dataFree = true;

    expectedPackets = (reading_length+FRAME_SIZE-1)/(FRAME_SIZE);
    waiting = 0;
    rdyToTX();
  }
  else {
    ackPacket.signature = createSignature(ACKK,ackPacket.signature>>2 + random(5) );  // I've got more, so try to take control.
    sendAck();
    waiting = MAX_WAIT_CHECKS;
    updateTXBuffer();
  }
}

#define DATA_SEND_DELAY 1
void readAckReq() {
  Ack_req readAckReq = {};
  memcpy(&readAckReq,RF_Rx_Buffer, RF_PACK_SIZE);

  unsigned char myPacketCount = ackPacket.signature>>2;
  if (myPacketCount==(readAckReq.signature>>2)) {
    if (ackPacket.package_size>14 && (readAckReq.low_mask>0 || readAckReq.high_mask>0)) {
      unsigned int bytes_left = ackPacket.package_size-14;
      if (!dataFree) {  // we are already expecting packets, retransmission.
        unsigned int current_mask = readAckReq.low_mask;
        unsigned int bitMask = 1;
        for (byte packetOn = 0;packetOn<expectedPackets;packetOn++) {
          if (current_mask & bitMask) {
            sendDataPacket(packetOn);  // re-transmit
            updateTXBuffer();
            delay(DATA_SEND_DELAY);
          }
          bitMask = bitMask<<1;
          if (bitMask==0) {
            bitMask=1;
            current_mask = readAckReq.high_mask;
          }
        }
      }
      else
      {
        expectedPackets = bytes_left+FRAME_SIZE-1;
        expectedPackets = expectedPackets >> 4;
        
        dataFree = false; // We are now going to use the datapackets
        
        for(byte packetOn = 0;packetOn<expectedPackets;packetOn++) {
          byte packetDataPtr = 0;
          dataPackets[packetOn].signature = createSignature(DATA,packetOn);
          while (bytes_left>0 && packetDataPtr<FRAME_SIZE) {
            bytes_left--;
            dataPackets[packetOn].data[packetDataPtr] = getTXbyte();
            packetDataPtr++;
          }
          while (packetDataPtr<FRAME_SIZE) {  // fills last buffer with 0s.
            dataPackets[packetOn].data[packetDataPtr] = 0;
            packetDataPtr++;
          }
          sendDataPacket(packetOn);
          updateTXBuffer();
          delay(DATA_SEND_DELAY);
        }
      }
    }
    currentState = FIN;
    sendFin(true,hasTXData() );
    waiting = MAX_WAIT_CHECKS;
    waiting = waiting>>1;
    retryCount = 0;  // This is a FIN retry.
  }
  else {
    sendAck();  // there was an error.  Re-send ACK.
    waiting = MAX_WAIT_CHECKS;
  }
    
  updateTXBuffer();
}

boolean canDisplayPacket(byte packetOn) {
  if (packetOn==0) return dataPackets[packetOn].signature>0;
  return dataPackets[packetOn].signature>0 && canDisplayPacket(packetOn-1);  // All previous packets are displayed.
}

void checkAndDisplay(byte packetOn) {
  if (packetOn>=expectedPackets) return;  // don't go above the packet boundry
  if (canDisplayPacket(packetOn)) {
      unsigned int remaining = reading_length & 15;
      if (packetOn+1<expectedPackets) {
        remaining = FRAME_SIZE;
      }
      for (byte i=0;i<remaining;i++) {
        Serial.write((dataPackets[packetOn]).data[i]);
      }
      checkAndDisplay(packetOn+1);
  }
}

void readData() {
  byte packetOn = RF_Rx_Buffer[0] >> 2;
  
  memcpy(&dataPackets[packetOn],RF_Rx_Buffer, RF_PACK_SIZE);
  
  checkAndDisplay(packetOn);
  
}

void readFin() {
  Fin readFin = {};
  memcpy(&readFin,RF_Rx_Buffer, RF_PACK_SIZE);
  
  boolean fromMaster = (readFin.signature>>2) & 1;
  boolean hasMore = (readFin.signature>>3) & 1;
  
  
  if (fromMaster && expectedPackets>0 && !canDisplayPacket(expectedPackets-1)) {
    sendAckReq();
    updateTXBuffer();
    return;
  }
  
  // Cleanup
  ackReqPacket.signature = 0;
  ackPacket.signature = 0;
  ackPacket.package_size = 0;

  dataFree = true;
  expectedPackets = 0;
  
  retryCount = 0;  // This is a FIN retry.
  
  if (!fromMaster) {
    currentState = ACKK;
    waiting = 0;
    if (hasMore && readFin.more_size>getTXBuffSize()) {
      currentState = FIN;
      master = false;
      sendFin(0,0);
      waiting = MAX_WAIT_CHECKS;
      waiting = waiting>>1;
      updateTXBuffer();
    }
    return;
  }

  currentState = FIN;
  waiting = MAX_WAIT_CHECKS;
  waiting = waiting>>1;
  master = !hasMore && hasTXData();
  transmit();  // Will send a FIN packet
  
  if (!master) {
    currentState=ACKK;
    waiting = 0;
  }
}



void setup() {   
       
        //LED and other interfaces
        pinMode(RED_LED_pin, OUTPUT); //RED LED
        pinMode(BLUE_LED_pin, OUTPUT); //GREEN LED
       
        //EEPROM check and update
        #if (PC_CONFIGURATION_ENABLED==1)
        eeprom_check(); 
        #endif
        
        //RF module pins
        pinMode(SDO_pin, INPUT); //SDO
        pinMode(SDI_pin, OUTPUT); //SDI        
	pinMode(SCLK_pin, OUTPUT); //SCLK
        pinMode(IRQ_pin, INPUT); //IRQ
        pinMode(nSel_pin, OUTPUT); //nSEL
        
        
        pinMode(0, INPUT); // Serial Rx
        pinMode(1, OUTPUT);// Serial Tx
        
           
        Serial.begin(SERIAL_BAUD_RATE);
        
     
      
         TCCR1B   =   0x00;   //stop timer
         TCNT1H   =   0x00;   //setup
         TCNT1L   =   0x00;
         ICR1     =   60005;   // used for TOP, makes for 50 hz
         TCCR1A   =   0x02;   
         TCCR1B   =   0x1A; //start timer with 1/8 prescaler for measuring 0.5us PPM resolution
         
         attachInterrupt(IRQ_interrupt,RFM22B_Int,FALLING);
}


//############ MAIN LOOP ##############
void loop() {
    
  unsigned char i;
  
  //wdt_enable(WDTO_1S);
  
  RF22B_init_parameter(); 
  frequency_configurator(CARRIER_FREQUENCY); // Calibrate the RF module for this frequency, frequency hopping starts from this frequency.
  
  Power_Set(7); // 7 = 100mW maximum RF output power
  
  Red_LED_ON ;
  Blue_LED_ON;
  delay(1000);	
  
  Red_LED_OFF;
  Blue_LED_OFF;
  
  transmitted = 0;
   to_rx_mode(); 
   sei();
   RF_Mode = Receive;
  	
  time = millis();
  old_time = time;
  
  ClearBuffers();
  
  while(1)
  
      {    /* MAIN LOOP */	              
      time = millis();  
      if (_spi_read(0x0C)==0) // detect the locked module and reboot
         {
         Red_LED_ON;  
         RF22B_init_parameter();
         frequency_configurator(CARRIER_FREQUENCY);
         rx_reset;
         Red_LED_OFF;
         }
      
       if (RF_Mode == Received)
         {
          Blue_LED_ON;  
          send_read_address(0x7f); // Send the package read command
	  for(i = 0; i<RF_PACK_SIZE; i++) //read all buffer 
            { 
            RF_Rx_Buffer[i] = read_8bit_data(); 
            }  
          rx_reset(); 
          
          RF_Mode = Receive;

          updateTXBuffer();
          readPacket();
          
	  for(i = 0; i<RF_PACK_SIZE; i++) //read all buffer 
            { 
            RF_Rx_Buffer[i] = 0;
            }  

          Blue_LED_OFF;
         }	    
      else
      {
        rdyToTX();
      }
  }
}


